﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace TBmemo
{
    public partial class Form1 : Form
    {
        string filePath = null;
        SearchDialogBox sdb = null;

        Bitmap _bgImg = null;

        public Bitmap bgImg
        {
            get { return _bgImg; }
            set { _bgImg = value; }
        }

        public TextBox TBox
        {
            get { return textBox1; }
        }


        
        public enum SearRepMode { Search, Replace };
        
        public Form1()
        {
            InitializeComponent();
        }

        #region File Menu Methods

        //終了
        private void CloseToolStripMenuItem_Click(object sender, EventArgs e)
        {           
                this.Close();
        }

        //名前を付けて保存
        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            saveFileDialog1.ShowDialog();            
        }

        private void saveFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            try
            {
                filePath = saveFileDialog1.FileName;
                File.WriteAllText(filePath, textBox1.Text, Encoding.Default);
                textBox1.Modified = false;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //開く
        private void OpenToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            bool b = DeleteTextAnnounce();

            if(b)
                openFileDialog1.ShowDialog();
        }

        private void openFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            try
            {
                filePath = openFileDialog1.FileName;
                textBox1.Text = File.ReadAllText(openFileDialog1.FileName, Encoding.Default);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void textBox1_ModifiedChange(object sender, EventArgs e)
        {
            if (textBox1.Modified == true)
                OverwriteToolStripMenuItem.Enabled = true;
            else
                OverwriteToolStripMenuItem.Enabled = false;
        }

        //上書き保存
        private void OverwriteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (filePath != null)
            {
                try
                {
                    File.WriteAllText(filePath, textBox1.Text, Encoding.Default);
                    textBox1.Modified = false;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.ToString(), "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
                saveFileDialog1.ShowDialog();
        }

        //新規
        private void NewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            bool b = DeleteTextAnnounce();
            if (b)
            {
                textBox1.Clear();
                textBox1.Modified = false;
                filePath = null;
            }           
        }

        private bool DeleteTextAnnounce()
        {
            if (textBox1.Modified)
            {
                DialogResult result;
                string msg = "変更内容は保存されていません。破棄してよろしいでしょうか？";
                result = MessageBox.Show(msg, "警告", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation);
                if (result == DialogResult.Yes)
                    return true;
                else
                    return false;
            }
            else            
                return true;
            
        }

        private void form1_FromClosing(object sender, FormClosingEventArgs e)
        {
            bool b = DeleteTextAnnounce();
            if(!b)
                e.Cancel = true;
        }

        #endregion

        #region Edit Menu Methods

        //切り取り
        private void CutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if(textBox1.SelectionLength > 0)
                textBox1.Cut();
        }

        //コピー
        private void CopyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (textBox1.SelectionLength > 0)
                textBox1.Copy();
        }

        //すべて選択
        private void AllSelectToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if(textBox1.Text.Length == textBox1.SelectionLength)
                textBox1.SelectionLength = 0;
            else
                textBox1.SelectAll();            
        }

        //元に戻す
        private void UndoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (textBox1.CanUndo)
            {
                textBox1.Undo();
                textBox1.ClearUndo();
            }
        }

        //貼り付け
        private void PasteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (Clipboard.GetDataObject().GetDataPresent(DataFormats.Text))
                textBox1.Paste();
        }

        //検索
        private void SearchToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (sdb == null || sdb.IsDisposed)
            {
                sdb = new SearchDialogBox(SearRepMode.Search, textBox1);
                this.AddOwnedForm(sdb);
                if (this.TopMost)
                    sdb.TopMost = true;
                sdb.Show();
            }
        }

        //置換
        private void ReplaceToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (sdb == null || sdb.IsDisposed)
            {
                sdb = new SearchDialogBox(SearRepMode.Replace, textBox1);
                this.AddOwnedForm(sdb);
                if (this.TopMost)
                    sdb.TopMost = true;
                sdb.Show();                
            }
        }

        //削除
        private void DeleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (textBox1.SelectionLength > 0)
            {
                int temp = textBox1.SelectionStart;
                textBox1.Text = textBox1.Text.Remove(textBox1.SelectionStart, textBox1.SelectionLength);
                textBox1.SelectionStart = temp;
                textBox1.Modified = true;
            }
        }

        //削除,切り取り,コピーの有効/無効の切り替え
        private void menuStrip1_MenuActivate(object sender, EventArgs e)
        {
            if (textBox1.SelectionLength > 0)
            {
                DeleteToolStripMenuItem.Enabled = true;
                CutToolStripMenuItem.Enabled = true;
                CopyToolStripMenuItem.Enabled = true;
            }
            else
            {
                DeleteToolStripMenuItem.Enabled = false;
                CutToolStripMenuItem.Enabled = false;
                CopyToolStripMenuItem.Enabled = false;
            }
        }       

        #endregion

        #region Form Menu Methods

        //右端で折り返す
        private void WordWrapToolStripMenuItem_Click(object sender, EventArgs e)
        {
            textBox1.WordWrap = !textBox1.WordWrap;
        }

        //フォント
        private void FontToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DialogResult dr = fontDialog1.ShowDialog();

            if (dr == DialogResult.OK)
            {
                textBox1.Font = fontDialog1.Font;
            }
        }

        #endregion

        #region View Menu Methods

        //ステータスバー
        private void statusbarToolStripMenuItem_Click(object sender, EventArgs e)
        {
            statusStrip1.Visible = !statusStrip1.Visible;

            if (statusStrip1.Visible)
            {
                textBox1.Size = new Size(textBox1.Width, textBox1.Height - 23);
                textBox1.Dock = DockStyle.None;
                textBox1.Anchor = (AnchorStyles.Bottom | AnchorStyles.Left |
                    AnchorStyles.Right | AnchorStyles.Top);
            }
            else
            {
                textBox1.Dock = DockStyle.Fill;
            }
        }                       

        //キーが押されたときのイベントハンドラ
        private void textBox1_KeyUp(object sender, KeyEventArgs e)
        {
            StatusLabelChange();

            CountLetter();
        }

        //ステータスバーの行、列ラベルの変更
        private void StatusLabelChange()
        {
            //行の変更            
            ToolStripStatusLabel tss = rowToolStripStatusLabel;
            tss.Text = (textBox1.GetLineFromCharIndex(textBox1.SelectionStart) + 1).ToString() + "行、";
            
            //列の変更
            tss = columnToolStripStatusLabel;
            int temp = textBox1.Text.LastIndexOf("\n", textBox1.SelectionStart);
            int column = textBox1.SelectionStart - temp;
            tss.Text = column.ToString() + "列";
        }

        //テキストボックスをクリックしたときのイベントハンドラ
        private void textBox1_Click(object sender, EventArgs e)
        {
            StatusLabelChange();
        }

        //文字カウント処理
        private void CountLetter()
        {
            int count;
            count = textBox1.Text.Length
                - textBox1.GetLineFromCharIndex(textBox1.Text.Length) * 2;
           
            
            countToolStripStatusLabel.Text = "文字数:" + count;
        }

        #endregion

        //常に手前表示
        private void TopMostToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.TopMost = !this.TopMost;
        }

        //オプション
        private void OptionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Option option = new Option(this);
            if (this.TopMost)
                option.TopMost = true;
            DialogResult dr = option.ShowDialog();

            if (dr == DialogResult.OK)
                option.SetOption();
        } 

    }
}
